import numpy as np
import scipy.stats as st

from uqtils.sobol import ishigami, sobol_sa


def test_sobol_sa():
    model = lambda x: ishigami(x)['y']
    n_samples = 2000

    def lhc_sampler(shape):
        N, dim = np.prod(shape), 3
        x = st.qmc.LatinHypercube(d=dim).random(n=N)
        x = st.qmc.scale(x, [-np.pi] * dim, [np.pi] * dim).reshape(shape + (dim,))
        return x

    # rand_sampler = lambda shape: np.random.rand(*shape, 3) * (2 * np.pi) - np.pi
    S1, S2, ST, bar_plot, pie_plot = sobol_sa(model, lhc_sampler, n_samples, cmap='summer', plot=True, compute_s2=True)

    S1_est = np.mean(S1, axis=0)[:, 0]
    S1_se = np.sqrt(np.var(S1, axis=0) / n_samples)[:, 0]
    S1_truth = np.array([0.3139, 0.4424, 0])

    S1_lb, S1_ub = S1_est - 2.5*S1_se, S1_est + 2.5*S1_se
    assert np.all(np.logical_and(S1_truth <= S1_ub, S1_truth >= S1_lb))



import numpy as np

from uqtils.example import gradient_example, mcmc_example, normal_example, sobol_example
from uqtils.uq_types import format_input


def test_all_examples():
    normal_example()
    gradient_example()
    mcmc_example()
    sobol_example()


def test_format_inputs():
    test_cases = [1.1, np.array(2.1), np.random.rand(2, 1), np.random.rand(1, 2), np.random.rand(3, 3)]

    for array in test_cases:
        ndim = np.atleast_1d(array).shape[-1]
        _, x = format_input(array, ndim)
        assert len(x.shape) == 2
        assert x.shape[-1] == ndim



"""Assorted utilities for uncertainty quantification and scientific computing.

- Author - Joshua Eckels (eckelsjd.@umich.edu)
- License - GPL-3.0

Includes:

- **MCMC** - A standard DRAM MCMC sampler.
- **Gradients** - Vectorized finite-difference implementation of Jacobian and Hessians.
- **Plotting** - Some plotting utilities for `matplotlib`.
- **Sobol'** - Sobol' global, variance-based sensitivity analysis.
"""
from .gradient import *  # noqa: F403
from .mcmc import *  # noqa: F403
from .plots import *  # noqa: F403
from .sobol import *  # noqa: F403
from .uq_types import *  # noqa: F403

__version__ = "0.3.1"
__all__ = gradient.__all__ + mcmc.__all__ + plots.__all__ + sobol.__all__ + uq_types.__all__  # noqa: F405
